﻿/*
 * Copyright (c) HiSilicon (Shanghai) Technologies Co., Ltd. 2024-2024. All rights reserved.
 * Description: HiLink产品适配实现源文件（此文件为DEMO，需集成方适配修改）
 */
#include "hilink_device.h"
#include <stdlib.h>
#include <stdbool.h>

#include "hilink.h"
#include "securec.h"
#include "hilink_sal_defines.h"
#include "cJSON.h"
#include "hsf.h"
#include "ble_cfg_net_api.h"
#include "cmsis_os2.h"
#include "device_profile.h"
#include "hilink_network_adapter.h"
#include "hilink_device.h"
#ifdef CONFIG_SUPPORT_HILINK_INDIE_UPGRADE
#include "hilink_entry.h"

extern void handle_device_online(void);
extern void handle_device_unbind(void);
extern void handle_device_offline(void);

// 声明外部函数
#include "switch_panel/switch_panel.h"

/*
 *
 * 服务信息定义
 * 注意：(1)适配格式{svcType， svcId}
 *       (2)与devicepartner平台物模型定义必须保持一致
 */
// 四开关面板服务信息定义
static const HILINK_SvcInfo SVC_INFO[] = {
    { "switch", "switch3" },
    { "switch", "switch2" },
    { "switch", "switch1" },
    { "switch", "switch" },
    { "switch", "switch4" },
#ifdef CONFIG_SUPPORT_HILINK_INDIE_UPGRADE
    //{ "checkSum", "checkSum" },
#endif
};

int HILINK_GetDevInfo(HILINK_DevInfo *devinfo)
{
    if (devinfo == NULL) {
        return -1;
    }
    int err = EOK;
    err = strcpy_s(devinfo->sn, sizeof(devinfo->sn), hfget_hilink_sn());
    if (err != EOK) {
        HILINK_SAL_ERROR("strcpy_s err:%d\r\n", err);
        return -1;
    }
    HILINK_SAL_DEBUG("HILINK_GetDevInfo sn: [%s]\r\n", hfget_hilink_sn());
    err = strcpy_s(devinfo->prodId, sizeof(devinfo->prodId), ProductId);
    if (err != EOK) {
        HILINK_SAL_ERROR("strcpy_s err:%d\r\n", err);
        return -1;
    }
    HILINK_SAL_DEBUG("HILINK_GetDevInfo prodId: [%s]\r\n", devinfo->prodId);
    err = strcpy_s(devinfo->subProdId, sizeof(devinfo->subProdId), "");
    if (err != EOK) {
        HILINK_SAL_ERROR("strcpy_s err:%d\r\n", err);
        return -1;
    }
    HILINK_SAL_DEBUG("HILINK_GetDevInfo subProdId: [%s]\r\n", devinfo->subProdId);
    err = strcpy_s(devinfo->model, sizeof(devinfo->model), deviceModel);
    if (err != EOK) {
        HILINK_SAL_ERROR("strcpy_s err:%d\r\n", err);
        return -1;
    }
    HILINK_SAL_DEBUG("HILINK_GetDevInfo model: [%s]\r\n", devinfo->model);
    err = strcpy_s(devinfo->devTypeId, sizeof(devinfo->devTypeId), deviceTypeId);
    if (err != EOK) {
        HILINK_SAL_ERROR("strcpy_s err:%d\r\n", err);
        return -1;
    }
    HILINK_SAL_DEBUG("HILINK_GetDevInfo devTypeId: [%s]\r\n", devinfo->devTypeId);
    err = strcpy_s(devinfo->devTypeName, sizeof(devinfo->devTypeName), configName);
    if (err != EOK) {
        HILINK_SAL_ERROR("strcpy_s err:%d\r\n", err);
        return -1;
    }
    HILINK_SAL_DEBUG("HILINK_GetDevInfo devTypeName: [%s]\r\n", devinfo->devTypeName);
    err = strcpy_s(devinfo->manuId, sizeof(devinfo->manuId), manufacturerID);
    if (err != EOK) {
        HILINK_SAL_ERROR("strcpy_s err:%d\r\n", err);
        return -1;
    }
    HILINK_SAL_DEBUG("HILINK_GetDevInfo manuId: [%s]\r\n", devinfo->manuId);
    err = strcpy_s(devinfo->manuName, sizeof(devinfo->manuName), configType);
    if (err != EOK) {
        HILINK_SAL_ERROR("strcpy_s err:%d\r\n", err);
        return -1;
    }
    HILINK_SAL_DEBUG("HILINK_GetDevInfo manuName: [%s]\r\n", devinfo->manuName);
#ifdef CONFIG_SUPPORT_HILINK_INDIE_UPGRADE
    err = sprintf_s(devinfo->fwv, sizeof(devinfo->fwv), "%s_%s_%s", "103","1.312", FIRMWARE_VER);
    if (err <= 0) {
        HILINK_SAL_ERROR("sprintf_s err:%d\r\n", err);
        return -1;
    }
#else
    err = strcpy_s(devinfo->fwv, sizeof(devinfo->fwv), FIRMWARE_VER);
    if (err != EOK) {
        HILINK_SAL_ERROR("strcpy_s err:%d\r\n", err);
        return -1;
    }
#endif
    HILINK_SAL_DEBUG("HILINK_GetDevInfo fwv: [%s]\r\n", devinfo->fwv);
    err = strcpy_s(devinfo->hwv, sizeof(devinfo->hwv), HARDWARE_VER);
    if (err != EOK) {
        HILINK_SAL_ERROR("strcpy_s err:%d\r\n", err);
        return -1;
    }
    HILINK_SAL_DEBUG("HILINK_GetDevInfo hwv: [%s]\r\n", devinfo->hwv);
    err = strcpy_s(devinfo->swv, sizeof(devinfo->swv), SOFTWARE_VER);
    if (err != EOK) {
        HILINK_SAL_ERROR("strcpy_s err:%d\r\n", err);
        return -1;
    }
    HILINK_SAL_DEBUG("HILINK_GetDevInfo swv: [%s]\r\n", devinfo->swv);
    return 0;
}

int HILINK_GetSvcInfo(HILINK_SvcInfo *svcInfo[], unsigned int size)
{
    unsigned int svcNum = sizeof(SVC_INFO) / sizeof(HILINK_SvcInfo);
    if ((svcInfo == NULL) || (size == 0) || (size < svcNum)) {
        return -1;
    }

    for (unsigned int i = 0; i < svcNum; ++i) {
        if (memcpy_s(svcInfo[i], sizeof(HILINK_SvcInfo), &SVC_INFO[i], sizeof(HILINK_SvcInfo)) != EOK) {
            return -1;
        }
    }
    e_printf("HILINK_GetSvcInfo svcNum:%d\r\n", svcNum);
    return svcNum;
}

/* AC参数 */
unsigned char A_C[48] = {
	    0x66, 0x22, 0x33, 0x5F, 0x75, 0x31, 0x29, 0x6A, 0x27, 0x34, 0x57, 0x44, 0x32, 0x42, 0x27, 0x59, 0xE1, \
	    0xDE, 0x9C, 0xA6, 0xA6, 0x77, 0x54, 0x8C, 0xA9, 0xB0, 0x0D, 0xC7, 0xCB, 0x1B, 0x32, 0x2B, 0xB9, 0x4E, \
	    0xEE, 0x97, 0xDC, 0x43, 0xFB, 0xF1, 0x86, 0xCC, 0xFD, 0x6E, 0x78, 0xA8, 0x36, 0x22};

/* 获取加密 AC 参数  */
unsigned char *HILINK_GetAutoAc(void)
{
    return A_C;
}


extern int HILINK_HILINK_Printf(const char *format, ...);


// 服务处理函数定义
typedef int (*handle_put_func)(const char* svc_id, const char* payload, unsigned int len);
typedef int (*handle_get_func)(const char* svc_id, const char* in, unsigned int in_len, char** out, unsigned int* out_len);
// 服务注册信息定义
typedef struct{
 // service id
 char* sid;
 // HILINK_PutCharState cmd function
 handle_put_func putFunc;
 // handle HILINK_GetCharState cmd function
 handle_get_func getFunc;
} HANDLE_SVC_INFO;
//不支持 HILINK_PutCharState 时，默认实现
int not_support_put(const char* svc_id, const char* payload, unsigned int len)
{
	 HILINK_Printf("sid:%s NOT SUPPORT PUT function \r\n", svc_id);
	 return 0;
}
// 服务处理函数
HANDLE_SVC_INFO g_device_profile[] = {
	{"switch", handle_put_switch, handle_get_switch},
    {"switch1", handle_put_switch1, handle_get_switch1},
    {"switch2", handle_put_switch2, handle_get_switch2},
    {"switch3", handle_put_switch3, handle_get_switch3},
    {"switch4", handle_put_switch4, handle_get_switch4},
    // 故障不支持 HILINK_PutCharState，配置 not_support_put
};
// 服务总数量
int g_device_profile_count = sizeof(g_device_profile) / sizeof(HANDLE_SVC_INFO);

// 辅助函数，用于查找服务注册信息
static HANDLE_SVC_INFO* find_handle(const char* svc_id)
{
	 for(int i = 0; i < g_device_profile_count; i++) {
	 HANDLE_SVC_INFO handle = g_device_profile[i];
	 if(strcmp(handle.sid, svc_id) == 0) {
		return &g_device_profile[i];
	 }
	 }
	 return NULL;
} 

// 分发设备控制指令
int handle_put_cmd(const char* svc_id, const char* payload, unsigned intlen)
{
	 HANDLE_SVC_INFO* handle = find_handle(svc_id);
	 if(handle == NULL) {
	 HILINK_Printf("no service to handle put cmd : %s \r\n", svc_id);
	 return -1;
	 }
	 handle_put_func function = handle->putFunc;
	 if(function == NULL) {
	 HILINK_Printf("put function is null for %s \r\n", svc_id);
	 return -1;
	 }
	 return function(svc_id, payload, intlen);
}
// 分发服务查询直连
int handle_get_cmd(const char* svc_id, const char* in, unsigned int in_len, char** out, unsigned int* out_len)
{
	 HANDLE_SVC_INFO* handle = find_handle(svc_id);
	 if(handle == NULL) {
	 HILINK_Printf("no service to handle get cmd : %s \r\n", svc_id);
	 return -1;
	 }
	 handle_get_func function = handle->getFunc;
	 if(function == NULL) {
	 HILINK_Printf("get function is null for %s \r\n", svc_id);
	 return -1;
	 }
	 return function(svc_id, in, in_len, out,out_len);
}

// 快速上报函数，用于上报服务状态信息
int fast_report(const char* svc_id)
{
	 char* payload = NULL;
	 int len;
	 int err = handle_get_cmd(svc_id, NULL, 0, &payload, (unsigned int *)&len);
	 if(err != 0) {
	 HILINK_Printf("get msg from %s failed \r\n", svc_id);
	 return err;
	 }
	 err = HILINK_ReportCharState(svc_id, payload, len);
	 HILINK_Printf("report %s result is %d, payload:%s \r\n", svc_id, err, payload);
	 return err;
} 
/*
 * 修改服务当前字段值
 * svcId为服务的ID，payload为接收到需要修改的Json格式的字段与其值，len为payload的长度
 * 返回0表示服务状态值修改成功，不需要底层设备主动上报，由Hilink Device SDK上报；
 * 返回-101表示获得报文不符合要求；
 * 返回-111表示服务状态值正在修改中，修改成功后底层设备必须主动上报；
 */
int HILINK_PutCharState(const char *svcId, const char *payload, unsigned int len)
{
    e_printf("[HILINK_PutCharState] 收到控制指令: svcId=%s, payload=%s\r\n", svcId, payload);
    
    if ((svcId == NULL) || (payload == NULL)) {
        return -1;
    }

    cJSON *json = cJSON_Parse(payload);
    if (json == NULL) {
        return -1;
    }
    cJSON_Delete(json);
    
    int err = handle_put_cmd(svcId, payload, len);
    e_printf("[HILINK_PutCharState] 控制指令处理完成，返回值: %d\r\n", err);
    return err;
}
#ifdef CONFIG_SUPPORT_HILINK_INDIE_UPGRADE
static int GetFwvCheckSum(char **out, unsigned int *outLen)
{
    char fwvCheckSum[65] = { 0 };
    unsigned int size = sizeof(fwvCheckSum);
    int ret = read_app_bin_hash(fwvCheckSum, &size);
    if (ret < 0) {
        HILINK_SAL_NOTICE("GetFwvCheckSum get fwv checksum ret:%d\r\n", ret);
        return -1;
    }
    unsigned int mallocLen = 128;
    char *newData = (char *)malloc(mallocLen);
    if (newData == NULL) {
        HILINK_SAL_NOTICE("GetFwvCheckSum malloc fail\r\n");
        return -1;
    }
    ret = sprintf_s(newData, mallocLen, "{\"checkSum\": \"%s\"}", fwvCheckSum);
    if (ret <= 0) {
        free(newData);
        HILINK_SAL_NOTICE("GetFwvCheckSum snHILINK_Printf_s ret:%d\r\n", ret);
        return -1;
    }
    *out = newData;
    *outLen = ret;
    return 0;
}
#endif
/*
 * 添加单设备多服务批控服务接口
 * 1.使用场景：单产品需要在一个报文包中包含多个服务同时处理
 * 2.使用方法：
 * (1)厂商需要编写支持批控的H5页面，增加type = 4以及在action中可以包含一个或者以上的服务。
 * (2)厂商需要通过HILINK_EnableBatchControl(bool flag)接口打开或关闭批量功能，当前默认关闭。
 * 3.使用约束：
 * (1)payload为接收到报文的actions内容，为数组格式，len为actions内容的长度；详情请参考《指南》
 * (2)返回值判断：
 *     返回0表示服务状态值修改成功，不需要底层设备主动上报，由HiLink SDK上报；
 *     返回-101表示获得报文不符合要求；
 *     返回-111表示服务状态值正在修改中；
 */
int HILINK_ControlCharState(const char *payload, unsigned int len)
{
    return 0;
}

/*
 * 获取服务字段值
 * svcId表示服务ID。厂商实现该函数时，需要对svcId进行判断；
 * in表示接收到的Json格式的字段与其值；
 * inLen表示接收到的in的长度；
 * out表示保存服务字段值内容的指针,内存由厂商开辟，使用完成后，由Hilink Device SDK释放；
 * outLen表示读取到的payload的长度；
 * 返回0表示服务状态字段值获取成功，返回非0表示获取服务状态字段值不成功。
 */
int HILINK_GetCharState(const char *svcId, const char *in, unsigned int inLen, char **out, unsigned int *outLen)
{
	if ((svcId == NULL) || (out == NULL) || (outLen == NULL)) 
	{
        return -1;
    }
	#ifdef CONFIG_SUPPORT_HILINK_INDIE_UPGRADE
    if (strcmp(svcId, "checkSum") == 0) {
        return GetFwvCheckSum(out, outLen);
    }
	#endif
	int err = handle_get_cmd(svcId, in, inLen, out, outLen);
	return err; 
}

/*
 * 功能: 获取SoftAp配网PIN码
 * 返回: 返回10000000到99999999之间的8位数字PIN码, 返回-1表示使用HiLink SDK的默认PIN码
 * 注意: (1)安全认证要求，PIN码不能由sn、mac等设备固定信息生成
 *       (2)该接口实现需要与devicepartner平台匹配
 */
int HILINK_GetPinCode(void)
{
    return -1;
}

/*
 * 通知设备的状态
 * status表示设备当前的状态
 * 注意，此函数由设备厂商根据产品业务选择性实现
 * 注意: (1) 此函数由设备厂商根据产品业务选择性实现
 *      (2) 禁止在HILINK_NotifyDevStatus接口内回调HiLink SDK的对外接口
 */
void HILINK_NotifyDevStatus(int status)
{
    HILINK_SAL_DEBUG("HILINK_NotifyDevStatus status: %d\r\n", status);
    switch (status) {
        case HILINK_M2M_CLOUD_OFFLINE:
            /* 设备与云端连接断开，请在此处添加实现 */
#if defined(SUPPORT_MINIMALIST_NETCFG) || defined(SUPPORT_BLE_ANCILLAY_NETCFG)
			BLE_CfgNetAdvUpdate(NULL);
            BLE_CfgNetAdvCtrl(0);
            (void)BLE_CfgNetAdvCtrl(0xFFFFFFFF);
#endif
            handle_device_offline();
            break;
        case HILINK_M2M_CLOUD_ONLINE:
            /* 设备连接云端成功，请在此处添加实现 */
#if defined(SUPPORT_MINIMALIST_NETCFG) || defined(SUPPORT_BLE_ANCILLAY_NETCFG)
            BLE_CfgNetAdvUpdate(NULL);
            BLE_CfgNetAdvCtrl(0);
            (void)BLE_CfgNetAdvCtrl(0xFFFFFFFF);
#endif
            /* 设备连接云端成功，请在此处添加实现 */
            handle_device_online();  // 处理设备上线
            break;
        case HILINK_M2M_LONG_OFFLINE:
            /* 设备与云端连接长时间断开，请在此处添加实现 */
            break;
        case HILINK_M2M_LONG_OFFLINE_REBOOT:
            /* 设备与云端连接长时间断开后进行重启，请在此处添加实现 */
			HILINK_SAL_DEBUG("device offline long time reboot\r\n");
            break;
        case HILINK_UNINITIALIZED:
            /* HiLink线程未启动，请在此处添加实现 */
            break;
        case HILINK_LINK_UNDER_AUTO_CONFIG:
            /* 设备处于配网模式，请在此处添加实现 */
            break;
        case HILINK_LINK_CONFIG_TIMEOUT:
            /* 设备处于10分钟超时状态，请在此处添加实现 */
            break;
        case HILINK_LINK_CONNECTTING_WIFI:
            /* 设备正在连接路由器，请在此处添加实现 */
            break;
        case HILINK_LINK_CONNECTED_WIFI:
            /* 设备已经连上路由器，请在此处添加实现 */
			hf_set_wifi_state(1);
            break;
        case HILINK_M2M_CONNECTTING_CLOUD:
            /* 设备正在连接云端，请在此处添加实现 */
            break;
        case HILINK_LINK_DISCONNECT:
            /* 设备与路由器的连接断开，请在此处添加实现 */
			hf_set_wifi_state(0);
            break;
        case HILINK_DEVICE_REGISTERED:
            /* 设备被注册，请在此处添加实现 */
            break;
        case HILINK_DEVICE_UNREGISTER:
            /* 设备被解绑，请在此处添加实现 */
			handle_device_unbind();
            break;
        case HILINK_REVOKE_FLAG_SET:
            /* 设备被复位标记置位，请在此处添加实现 */
            handle_device_unbind();
            break;
        case HILINK_NEGO_REG_INFO_FAIL:
            /* 设备协商配网信息失败 */
            break;
        case HILINK_LINK_CONNECTED_FAIL:
            /* 设备与路由器的连接失败 */
            break;
        case HILINK_CLOUD_TO_CENTRAL:
            /* 直连云模式切成中枢模式 */
            break;
        case HILINK_DEVICE_CLEAN:
            /* 设备完全清除，包括前装跟后装信息 */
            break;
        default:
            break;
    }

    return;
}

 /*
 * 功能：实现模组重启前的设备操作
 * 参数：flag 入参，触发重启的类型
 *          0表示HiLink SDK 线程看门狗触发模组重启;
 *          1表示APP删除设备触发模组重启;
 *          2表示设备长时间离线无法恢复而重启;
 * 返回值：0表示处理成功, 系统可以重启，使用硬重启;
 *         1表示处理成功, 系统可以重启，如果通过HILINK_SetSdkAttr()注册了软重启(sdkAttr.rebootSoftware），使用软重启;
 *         负值表示处理失败，系统不能重启
 * 注意：(1) 此函数由设备厂商实现；
 *       (2) 若APP删除设备触发模组重启时，设备操作完务必返回0，否则会导致删除设备异常；
 *       (3) 设备长时间离线无法恢复而重启，应对用户无感，不可影响用户体验，否则不可以重启；
 */
int HILINK_ProcessBeforeRestart(int flag)
{
    /* HiLink SDK线程看门狗超时触发模组重启 */
    if (flag == 0) {
        /* 实现模组重启前的操作(如:保存系统状态等) */
        return 1;
    }

    /* APP删除设备触发模组重启 */
    if (flag == 1) {
        /* 实现模组重启前的操作(如:保存系统状态等) */
		handle_device_unbind();  // 处理设备解绑
        return 1;
    }

    /* 设备长时间离线触发模组重启，尝试恢复网络 */
    if (flag == 2) {
        /* 实现模组重启前的操作(如:保存系统状态等) */
        return 1;
    }

    return -1;
}
