#!/usr/bin/env python3
# encoding=utf-8
# ============================================================================
# @brief    Target Definitions File
# Copyright (c) HiSilicon (Shanghai) Technologies Co., Ltd. 2022-2023. All rights reserved.
# ============================================================================
target = {
    'ws63-liteos-app': {
        'base_target_name': 'target_ws63_app_rom_template',
        'os': 'liteos',
        'defines': [
            "USE_CMSIS_OS",
            "USE_LITEOS",
            "_ALL_SOURCE",
            "__LITEOS__",
            "-:CHIP_WS63=1",
            "LIBUTIL_COMPAT",
            "LOG_SUPPORT",
            "HSO_SUPPORT",
            "NO_TCXO_SUPPORT",
            "UNSUPPORT_OTHER_MEM",
            "SW_UART_DEBUG",
            "SW_UART_CHIP_DEFINE",
            "AT_COMMAND",
            "DUMP_MEM_SUPPORT",
            "BUILD_APPLICATION_STANDARD",
            'CMD_ENABLE',
            'WIFI_TASK_EXIST',
            'BGLE_TASK_EXIST',
            'BTH_TASK_EXIST',
            "CONFIG_IPERF_SUPPORT",
            "CONFIG_SENDTEST_SUPPORT",
            "_PRE_WLAN_FEATURE_BTCOEX",
            "_PRE_LWIP_ZERO_COPY",
            #"WIFI_TCM_OPTIMIZE",
            #"LWIP_TCM_OPTIMIZE",
            "CHECKSUM_CHECK_TCP=0",
            "CHECKSUM_CHECK_UDP=0",
            "MBEDTLS_AES_ROM_TABLES",
            "UPDATE_WIFI_STATIC_LIB",
            "UPDATE_BTC_STATIC_LIB",
            "CONFIG_NV_SUPPORT_SINGLE_CORE_SYSTEM", 'CONFIG_OTA_UPDATE_SUPPORT',
            "MBEDTLS_HARDEN_OPEN",
            'CONFIG_UART_SUPPORT_LPM',
            "_PRE_RADAR_CCA_SW_OPT",
            "MBEDTLS_CONFIG_FILE=\"config-ws-iot.h\"",
            "CONFIG_NO_VERIFY_TLS_TIME",
            "CHBA_SUPPORT",
            "_PRE_WLAN_FEATURE_SLE_BRIDGE",
        ],
        'ram_component': [
            'ws63_liteos_app',
            'ws63_liteos_app_lds',
            'liteos_port',
            'irmalloc',
            'non_os',
            'arch_port',
            'board_config', # 被 pinctrl 组件依赖
            'lpm', # 被 pwm 组件依赖
            'chip_ws63', 'pmp_cfg_ws63',
            'reboot', 'hal_reboot', 'reboot_port', 'cpu_utils', 'hal_cpu_core',
            'gpio','hal_gpio_v150','gpio_port',
            "dfx_port_ws63", "algorithm", "cmn_header", "lwip", "lwip_tcm", "wifi_service", "mbedtls", "wpa_supplicant",
            "at", "wifi_driver_hmac", "wifi_driver_dmac", "wifi_driver_tcm", "wifi_at", "wifi_csa", "wifi_frag", "wifi_alg_txbf", "wifi_alg_temp_protect", "wifi_tx_amsdu",
            "wifi_auto_adjust_freq", "wifi_alg_anti_interference", "wifi_alg_edca_opt", "wifi_alg_cca_opt", "wifi_radar_sensor",
            'wifi_btcoex', "wifi_uapsd_ap", 'sio_port', 'i2s', 'hal_sio',
            'liteos_208_5_0',
            'nv', 'nv_ws63', 'nv_zdiag_ws63', 'plt_at', 'dfx_printer',
            'update_common', 'update_common_ws63', 'update_ab_ws63', 'factory_ws63', 'update_storage', 'update_storage_ws63',
            'pm_port_ws63',
            'gmssl_hmac_sm3',
            'bt_at',
            "bt_host",
            'bg_common',
            'bth_gle',
            'bth_sdk',
            'samples',
            'bts_header',
            'bt_app',
            'mips',
            'hal_mips',
            "bgtp",
            'soc_port',
            #'radar_sensing',
            #'radar_at',
            #'radar_ai',
            "cjson",
            'xo_trim_port',
            "coap",
            "sle_netdev"
        ],
        'ccflags': [
            "-DBOARD_ASIC", '-DPRE_ASIC',
        ],
        'application': 'application',
        'bin_name': 'ws63-liteos-app',
        'smaller': True,
        'hso_enable_bt': True,
        'hso_enable': True,
        'codesize_statistic': True,
        'nv_update':True,
        'generate_efuse_bin': True,
        'copy_files_to_interim': True
    },
    'ws63-flashboot': {
        'base_target_name': 'target_ws63_boot_template',
        'CONFIG_TIMER_USING_V150': 'y',
        "CONFIG_PMP_USING_RISCV_31" : 'y',
        'defines': [
            "__NON_OS__",
            "WS63_PRODUCT_NONE",
            "CONFIG_TIMER_MAX_NUM=3",
            "CONFIG_TIMER_CLOCK_VALUE=24000000",
            "CONFIG_TIMER_0_WIDTH_64=0",
            "SW_UART_DEBUG",
            "CONFIG_UART_SUPPORT_RX",
            "CONFIG_UART_SUPPORT_TX",
            "BUILD_APPLICATION_ROM",
            "BUILD_NOOSAL",
            "FLASH_REGION_CFG_FLASHBOOT",
            "EFUSE_BIT_OPERATION",
        ],
        'ram_component': [
            "flashboot_common", "common_boot", "ws63_flashboot_lds", "dfx_preserve",
            "libboundscheck", "chip_ws63", "ws63_mem_config", "common_headers","non_os",
            "sfc_port_ws63", "sfc_flash_config_ws63", "sfc_boot", "hal_sfc",
            "cmn_header", "arch_port", "osal", "dfx_panic", "dfx_exception","cpu_utils",
            '-:rtc', '-:hal_rtc', "common_boot_libc",
            "error_code", "board_config", "chip_boot_port", "chip_boot_config",
             '-:hal_systick', 'partition', 'partition_ws63','pmp_cfg_ws63', 'nonos_malloc', 'nonos_malloc_port',
            'update_common', 'update_local', 'update_local_ws63', 'lzma_22.00', 'update_storage', 'update_common_ws63', 'update_ab_ws63', 'factory_ws63',
            'efuse', 'hal_efuse_v151', 'efuse_port', 'soc_port',
        ],
        'ram_component_set': ['uart', "time_set", "cpu", "pinctrl", "watchdog", "security_unified",'pmp_set'],
        'os': 'non-os',
        'application': 'flashboot',
        'bin_name': 'flashboot'
    },
    'ws63-loaderboot': {
        'base_target_name': 'target_ws63_boot_template',
        'CONFIG_TIMER_USING_V150': 'y',
        'defines': [
            "WS63_PRODUCT_NONE",
            "CONFIG_TIMER_MAX_NUM=3",
            "CONFIG_TIMER_CLOCK_VALUE=24000000",
            "CONFIG_TIMER_0_WIDTH_64=0",
            "SW_UART_DEBUG",
            "CONFIG_UART_SUPPORT_RX",
            "CONFIG_UART_SUPPORT_TX",
            "BUILD_APPLICATION_ROM",
            "EFUSE_BIT_OPERATION",
            "PROVISION_WRITE_WITH_INTERFACE",
            "FLASH_REGION_CFG_LOADERBOOT",
            "BUILD_NOOSAL",
            "CONFIG_LOADERBOOT"
        ],
        'ram_component': [
            "loaderboot_common", "common_boot", "ws63_loaderboot_lds", "dfx_preserve",
            "libboundscheck", "chip_ws63", "ws63_mem_config", "common_headers","non_os",
            "sfc_port_ws63", "sfc_flash_config_ws63", "sfc_boot", "hal_sfc",
            "cmn_header", "arch_port", "osal", "dfx_panic", "dfx_exception","cpu_utils",
            '-:rtc', '-:hal_rtc', "common_boot_libc",'soc_port', 'nonos_malloc', 'nonos_malloc_port', 'boot_port_malloc',
            "error_code", "board_config", "chip_boot_port", "chip_boot_config",
             '-:hal_systick','efuse_port', 'hal_efuse_v151', 'efuse',
        ],
        'ram_component_set': ['uart', "time_set", "cpu", "pinctrl", "watchdog", "security_unified"],
        'os': 'non-os',
        'application': 'loaderboot',
        'bin_name': 'loaderboot'
    },
    'ws63-liteos-xts': {
        'base_target_name': 'target_ws63_xts_rom_template',
        'liteos_kconfig': 'ws63_xts',
        'os': 'liteos',
        'defines': [
            "USE_CMSIS_OS",
            "USE_LITEOS",
            "_ALL_SOURCE",
            "__LITEOS__",
            "-:CHIP_WS63=1",
            "LIBUTIL_COMPAT",
            "LOG_SUPPORT",
            "HSO_SUPPORT",
            "CONFIG_SUPPORT_NEW_DIAG",
            "NO_TCXO_SUPPORT",
            "UNSUPPORT_OTHER_MEM",
            "SW_UART_DEBUG",
            "SW_UART_CHIP_DEFINE",
            "AT_COMMAND",
            "DUMP_MEM_SUPPORT",
            "BUILD_APPLICATION_STANDARD",
            'CMD_ENABLE',
            'WIFI_TASK_EXIST',
            'BGLE_TASK_EXIST',
            'BTH_TASK_EXIST',
            "CONFIG_IPERF_SUPPORT",
            "CONFIG_SENDTEST_SUPPORT",
            "CONFIG_CACHE_MISS_SUPPORT",
            "_PRE_WLAN_FEATURE_BTCOEX",
            "_PRE_LWIP_ZERO_COPY",
            #"WIFI_TCM_OPTIMIZE",
            #"LWIP_TCM_OPTIMIZE",
            "CHECKSUM_CHECK_TCP=0",
            "CHECKSUM_CHECK_UDP=0",
            "MBEDTLS_AES_ROM_TABLES",
            "UPDATE_WIFI_STATIC_LIB",
            "UPDATE_BTC_STATIC_LIB",
            "CONFIG_NV_SUPPORT_SINGLE_CORE_SYSTEM", 'CONFIG_OTA_UPDATE_SUPPORT',
            "MBEDTLS_HARDEN_OPEN",
            'CONFIG_UART_SUPPORT_LPM',
            "_PRE_RADAR_CCA_SW_OPT",
        ],
        'ram_component': [
            'ws63_liteos_app',
            'ws63_liteos_xts_lds',
            'liteos_port',
            'irmalloc',
            'non_os',
            '-:rtc',
            'arch_port',
            'board_config', # 被 pinctrl 组件依赖
            'lpm', # 被 pwm 组件依赖
            'chip_ws63', 'pmp_cfg_ws63',
            'reboot', 'hal_reboot', 'reboot_port', 'cpu_utils', 'hal_cpu_core',
            'gpio','hal_gpio_v150','gpio_port',
            "dfx_port_ws63", "algorithm", "cmn_header", "lwip", "lwip_tcm", "wifi_service", "mbedtls", "wpa_supplicant",
            "at", "wifi_driver_hmac", "wifi_driver_dmac", "wifi_driver_tcm", "wifi_at", "wifi_csa", "wifi_alg_txbf", "wifi_alg_temp_protect", "wifi_tx_amsdu", "wifi_sdp",
            "wifi_auto_adjust_freq", "wifi_promisc", "wifi_alg_anti_interference", "wifi_alg_edca_opt", "wifi_alg_cca_opt", "wifi_sr",
            "wifi_frag", "wifi_mbo", "wifi_bsrp_nfrp", "wifi_slp", "wifi_radar_sensor", "wifi_apf", "wifi_repeater", "wifi_csi", "wifi_wapi", "wifi_wps", "wifi_psd", "wifi_blacklist", "wifi_m2u",
            "wifi_latency", "wifi_roam", "wifi_uapsd_sta", "wifi_ant_sel", "wifi_twt", "wifi_11k", "wifi_11v", "wifi_11r",  'wifi_btcoex', "wifi_uapsd_ap", "wifi_dnb",
            'liteos_208_5_0', 'rtc_unified', 'hal_rtc_unified', 'rtc_unified_port',
            'nv', 'nv_ws63', 'nv_zdiag_ws63', 'plt_at',
            'update_common', 'update_common_ws63', 'update_ab_ws63', 'factory_ws63', 'update_storage', 'update_storage_ws63',
            'pm_port_ws63',
            'gmssl_hmac_sm3',
            'bt_at',
            "bt_host",
            'bg_common',
            'bth_gle',
            'bth_sdk',
            'samples',
            'bts_header',
            'bt_app',
            'mips',
            'hal_mips',
            "bgtp",
            'soc_port',
            'radar_sensing',
            'radar_at',
            'ohos_adapt',
            'printf_adapt',
            'little_fs', 'littlefs_adapt_ws63',
            'xo_trim_port',
        ],
        'ccflags': [
            "-DBOARD_ASIC", '-DPRE_ASIC',
        ],
        'application': 'application',
        'bin_name': 'ws63-liteos-xts',
        'hso_enable_bt': True,
        'hso_enable': True,
        'codesize_statistic': True,
        'nv_update':True,
        'copy_files_to_interim': True
    },
    'ws63-liteos-hilink': {
        'base_target_name': 'target_ws63_hilink_rom_template',
        'os': 'liteos',
        'defines': [
            'CONFIG_SUPPORT_HILINK_INDIE_UPGRADE',
            'SUPPORT_QUICK_NETCFG',
        ],
        'ram_component': [
            'ws63_liteos_hilink_lds',
            'hilink_addr_map',
            'hilinkdevicesdk',
            'hilinkota',
            'hilinkbtsdk',
            'hilinkquickcfg',
        ],
        'ccflags': [
            "-DBOARD_ASIC", '-DPRE_ASIC',
        ],
        'application': 'application',
        'bin_name': 'ws63-liteos-hilink',
        'copy_files_to_interim': True
    },
    'ws63-liteos-app-iot': {
        'base_target_name': 'target_ws63_app_rom_template',
        # 'liteos_kconfig': 'ws63_iot',
        'os': 'liteos',
        'defines': [
            "USE_CMSIS_OS",
            "USE_LITEOS",
            "_ALL_SOURCE",
            "__LITEOS__",
            "-:CHIP_WS63=1",
            "LIBUTIL_COMPAT",
            "LOG_SUPPORT",
            "HSO_SUPPORT",
            "NO_TCXO_SUPPORT",
            "UNSUPPORT_OTHER_MEM",
            "SW_UART_DEBUG",
            "SW_UART_CHIP_DEFINE",
            "AT_COMMAND",
            "DUMP_MEM_SUPPORT",
            "BUILD_APPLICATION_STANDARD",
            'CMD_ENABLE',
            'WIFI_TASK_EXIST',
            'BGLE_TASK_EXIST',
            'BTH_TASK_EXIST',
            "CONFIG_IPERF_SUPPORT",
            "CONFIG_SENDTEST_SUPPORT",
            "_PRE_WLAN_FEATURE_BTCOEX",
            "_PRE_LWIP_ZERO_COPY",
            #"WIFI_TCM_OPTIMIZE",
            #"LWIP_TCM_OPTIMIZE",
            "CHECKSUM_CHECK_TCP=0",
            "CHECKSUM_CHECK_UDP=0",
            "MBEDTLS_AES_ROM_TABLES",
            "UPDATE_WIFI_STATIC_LIB",
            "UPDATE_BTC_STATIC_LIB",
            "CONFIG_NV_SUPPORT_SINGLE_CORE_SYSTEM", 'CONFIG_OTA_UPDATE_SUPPORT',
            "MBEDTLS_HARDEN_OPEN",
            'CONFIG_UART_SUPPORT_LPM',
            "MBEDTLS_CONFIG_FILE=\"config-ws-iot.h\"",
            "CONFIG_SUPPORT_HILINK",
            "_PRE_RADAR_CCA_SW_OPT",
            "SUPPORT_MINIMALIST_NETCFG",     # 极简配网
            "SUPPORT_SOFTAP_NETCFG",         # softAP配网
            "SUPPORT_BLE_ANCILLAY_NETCFG",   # ble辅助配网
            # "SUPPORT_QUICK_NETCFG",        # 快速配网
            "CONFIG_SUPPORT_HILINK_INDIE_UPGRADE",
            "CONFIG_DHCPS_GW",
            "_HSF_",
		  # "ENABLE_BLE_SCAN"    #open ble scan
        ],
        'ram_component': [
            'ws63_liteos_app',
            'ws63_liteos_app_lds',
            'liteos_port',
            'irmalloc',
            'non_os',
            '-:rtc',
            'arch_port',
            'board_config', # 被 pinctrl 组件依赖
            'lpm', # 被 pwm 组件依赖
            'chip_ws63', 'pmp_cfg_ws63',
            'reboot', 'hal_reboot', 'reboot_port', 'cpu_utils', 'hal_cpu_core',
            'wifi_promisc',
            'gpio','hal_gpio_v150','gpio_port',
            "dfx_port_ws63", "algorithm", "cmn_header", "lwip", "lwip_tcm", "wifi_service", "mbedtls", "wpa_supplicant",
            "at", "wifi_driver_hmac", "wifi_driver_dmac", "wifi_driver_tcm", "wifi_at", "wifi_csa", "wifi_frag", "wifi_alg_txbf", "wifi_alg_temp_protect", "wifi_tx_amsdu",
            "wifi_auto_adjust_freq", "wifi_alg_anti_interference", "wifi_alg_edca_opt", "wifi_alg_cca_opt", "wifi_radar_sensor",
            'wifi_btcoex', "wifi_uapsd_ap",
            'liteos_208_5_0', 'rtc_unified', 'hal_rtc_unified', 'rtc_unified_port',
            'nv', 'nv_ws63', 'nv_zdiag_ws63', 'plt_at', 'dfx_printer',
            'update_common', 'update_common_ws63', 'update_ab_ws63', 'factory_ws63', 'update_storage', 'update_storage_ws63',
            'pm_port_ws63',
            'gmssl_hmac_sm3',
            'bt_at',
            "bt_host",
            'bg_common',
            'bth_gle',
            'bth_sdk',
            'samples',
            'bts_header',
            'bt_app',
            'mips',
            'hal_mips',
            "bgtp",
            'soc_port',
            'radar_sensing',
            'radar_at',
            'cjson',
            'xo_trim_port',
            'hilink',
            'app_addr_map',
            # 'hilinkdevicesdk',
            # 'hilinkota',
            # 'hilinkbtsdk',
            'huks_sdk',
            'deviceauth',
            'little_fs', 'littlefs_adapt_ws63',
            'hsf',
            'user_main'
        ],
        'ccflags': [
            "-DBOARD_ASIC", '-DPRE_ASIC',
        ],
        'application': 'application',
        'bin_name': 'ws63-liteos-app-iot',
        'smaller': True,
        'hso_enable_bt': True,
        'hso_enable': True,
        'codesize_statistic': True,
        'nv_update':True,
        'copy_files_to_interim': True
    },
}

# custom copy rules, put it in target_group below and it takes effect.
# <root> means root path
# <out_root> means output_root path
# <pack_target> means target_group key_name
target_copy = {

}

target_group = {

}
